# -*- coding: utf8 -*-
from datetime import date, datetime
import math
import decimal

from ringo_bmasreport.views import durchfuehrungsbericht as evaluate_view
from ringo_bmasreport.reports.durchfuehrungsbericht import (
    filter_broken_participations
)
from efa.model.beneficiary import get_beneficiary_configuration_by_date


def prettify(date):
    return date.strftime("%d.%m.%Y")


def read_date(date_str):
    return datetime.strptime(date_str, "%Y-%m-%d").date()


def report_targetsystem(request, program, Participation, recipe,
                        validate, derive_indicators, indicators, filters):
    result = evaluate_view(request, program, Participation,
                           create_evaluate(request), recipe,
                           validate, derive_indicators, indicators, filters,
                           getvalidate=True)
    # the dates used in this view
    start = read_date(request.registry.settings["efa.frueherkennung.start"])
    end = read_date(request.registry.settings["efa.frueherkennung.end"])
    today = date.today()
    total_months = calculate_difference_in_months(start, end)
    actual_months = calculate_difference_in_months(start, today)
    quota_months = int(100 * float(actual_months) / float(total_months))

    # entries and exits
    entries = result["out"]["entries"]
    exits = result["out"]["exits"]
    exits_pi = result["out"]["exits_pi"]

    # I expect target_oi and target_ri to be integer here
    target_oi, target_ri = get_goals_from(request.db, start)
    goal = target_oi  # Zielwert OI Soll
    e_goal_percent = 0 if goal == 0 else 100 * entries[2] / goal  # OI in %
    fac = float(actual_months) / float(total_months)  # Anteil
    actual_e_goal = int(math.ceil(goal * fac))   # Zwischenziel OI Soll
    # Zwischenziel OI ist in Prozent
    actual_e_goal_perc = 0 if actual_e_goal == 0 else (
        100 * entries[2] / actual_e_goal)
    zuwendung = target_ri
    rel_perc = 0 if exits[2] == 0 else int(
        round(100 * float(exits_pi[2]) / float(exits[2])))
    rel_entries_perc = 0 if entries[2] == 0 else int(
        round(100 * float(exits_pi[2]) / float(entries[2])))
    indicator_text = request.registry.settings.get(
        "efa.frueherkennung_indicator_text").decode("utf-8")
    return {
        "program": program.upper(),
        "instance_id": get_beneficiary_id(request.db, start),
        "result": result["out"],
        "start": prettify(start),
        "end": prettify(end),
        "today": prettify(today),
        "total_months": total_months,
        "actual_months": actual_months,
        "quota_months": quota_months,
        "goal": goal,
        "e_goal_percent": e_goal_percent,
        "actual_e_goal": actual_e_goal,
        "actual_e_goal_perc": actual_e_goal_perc,
        "current_e_goal_perc_color": get_color(actual_e_goal_perc),
        "zuwendung": zuwendung,
        "rel_perc": rel_perc,
        "rel_entries_perc": rel_entries_perc,
        "indicator_text": indicator_text

    }


def get_color(value):
    if 0 <= value <= 64:
        return "FF4A4A"  # red
    elif value <= 84:
        return "FF924A"  # orange
    elif value <= 149:
        return "4AFF4A"  # green
    return "ffffff"  # white


def calculate_difference_in_months(start_date, end_date):
    """Calculates the difference between to dates
        end_date (date): Defaults to None. upper bound
        start_date (date): Defaults to None. [description]

    Every partial month is counted. 01.01.2017 - 01.01.2018
    is counted as 13 months.

    Returns:
        Integer: months
    """

    months_from_years = (end_date.year - start_date.year) * 12
    return months_from_years + end_date.month - start_date.month + 1


def get_beneficiary_id(db, start):
    beneficiary_configuration = get_beneficiary_configuration_by_date(
        db, start)
    return beneficiary_configuration.beneficiary_id


def complete_ri(participation):
    return participation.get("completeness_exit") == "100"


def sum_participations(collection, gender):
    return sum([1 for pn in collection if pn.get("gender") == gender])


def return_gender_participtions_tuple(collection):
    sum_m = sum_participations(collection, 1)
    sum_w = sum_participations(collection, 0)
    sum_total = sum_m + sum_w
    return (sum_w, sum_m, sum_total)


def create_evaluate(request):
    indicator = request.registry.settings.get("efa.frueherkennung_indicator")

    def project_specific(participation):
        return participation.get(indicator)

    def evaluate(participations,
                 start, end, filter_type,
                 oi_fields, ri_fields,
                 filters, foerderphase=None):
        participations = filter_broken_participations(participations)
        entries = participations
        exits = filter(complete_ri, participations)
        exits_pi = filter(project_specific, exits)
        # exits with programm indicator

        out = {}
        out['exits'] = return_gender_participtions_tuple(exits)
        out['entries'] = return_gender_participtions_tuple(entries)
        out['exits_pi'] = return_gender_participtions_tuple(exits_pi)
        out['start_date'] = start
        out['end_date'] = end
        return out
    return evaluate


def get_goals_from(database, start):
    """Gets goals from Database

    Business logic:

    ri_target_percent is a percentage value multiplied by 100.
    See scripts/frueherkennung/set_targets.py
    The multiplication needs an inverse operation.
    We round the result for readability reasons

    Args:
        database (Engine): DB-Connection


    Returns:
        tuple: oi_target_number, ri_target_percent
    """

    beneficiary = get_beneficiary_configuration_by_date(database, start)
    oi_target_number = beneficiary.oi_target_number
    ri_target_percent = beneficiary.ri_target_percent
    ri_target_percent = 0 if ri_target_percent == 0 else int(
        round(decimal.Decimal(ri_target_percent) / 100))
    return (oi_target_number, ri_target_percent)
